### R code from vignette source 'fundamentalFM.Rnw'
### Encoding: UTF-8

###################################################
### code chunk number 1: args
###################################################
library(factorAnalytics)
args(fitFundamentalFactorModel)


###################################################
### code chunk number 2: loading.data
###################################################
#equity <- data(equity)
equity <- read.csv(file="equity.csv")
names(equity)
length(unique(equity$datadate)) # number of period t 
length(unique(equity$tic)) # number of assets


###################################################
### code chunk number 3: get.returns
###################################################
library(quantmod)   # for Delt. See Delt for detail
equity <- cbind(equity,do.call(rbind,lapply(split(equity,equity$tic), 
                                            function(x) Delt(x$PRCCQ)))) 
names(equity)[22] <- "RET"


###################################################
### code chunk number 4: get.mv.bm
###################################################
equity$MV <- log(equity$PRCCQ*equity$CSHOQ) 
equity$BM <- equity$CEQQ/equity$MV


###################################################
### code chunk number 5: as.yearqtr
###################################################
a <- unlist( lapply(strsplit(as.character(equity$DATACQTR),"Q"),
               function(x)  paste(x[[1]],"-",x[[2]],sep="") ) )
equity$yearqtr <- as.yearqtr(a,format="%Y-%q")
equity$tic <- as.character(equity$tic)
equity <- subset(equity,yearqtr != "2000 Q1") # delete the first element of each assets


###################################################
### code chunk number 6: fit.ffm
###################################################
fit.fund <- fitFundamentalFactorModel(exposure.names=c("BM","MV"),datevar="yearqtr",
                                      returnsvar ="RET",assetvar="tic",wls=TRUE,
                                      data=equity)
names(fit.fund)


###################################################
### code chunk number 7: fit.fund
###################################################
names(fit.fund$returns.cov)


###################################################
### code chunk number 8: factor.return
###################################################
fit.fund$factor.returns
fit.fund$residuals
fit.fund$tstats


###################################################
### code chunk number 9: generic.fun
###################################################
summary(fit.fund)
predict(fit.fund)
print(fit.fund)


###################################################
### code chunk number 10: plot.fit.fund
###################################################
plot(fit.fund,which.plot=1,max.show=3)


###################################################
### code chunk number 11: fig1
###################################################
plot(fit.fund,which.plot=1,max.show=3)


###################################################
### code chunk number 12: ind.model
###################################################
equity$SPCINDCD <- as.character(equity$SPCINDCD)
fit.ind <- fitFundamentalFactorModel(exposure.names=c("SPCINDCD"),datevar="yearqtr",
                                      returnsvar ="RET",assetvar="tic",wls=FALSE,
                                     data=equity)


###################################################
### code chunk number 13: mixted
###################################################
fit.mix <- fitFundamentalFactorModel(exposure.names=c("BM","MV","SPCINDCD"),
                                     datevar="yearqtr",returnsvar ="RET",
                                     assetvar="tic",wls=FALSE,data=equity)


###################################################
### code chunk number 14: standardized.fund
###################################################
equity$weight <- sqrt(exp(equity$MV)) # we took log for MV before.


###################################################
### code chunk number 15: s.fit
###################################################
fit.fund2 <- fitFundamentalFactorModel(exposure.names=c("BM","MV"),
                                     datevar="yearqtr",returnsvar ="RET",
                                     assetvar="tic",wls=TRUE,data=equity,
                                     standardized.factor.exposure = TRUE,
                                       weight.var = "weight"  )


###################################################
### code chunk number 16: create.xts
###################################################
library(xts)
tic <- unique(equity$tic)
ret <- xts(NA,as.yearqtr("2000 Q2",format="%Y Q%q"))
for (i in tic) {
temp <- subset(equity,tic == i)
ret.new <- xts(temp$RET,as.yearqtr(temp$yearqtr))
names(ret.new) <- i
ret <- merge(ret,ret.new)
}
ret <- ret[,-1]
dim(ret)


###################################################
### code chunk number 17: stat.fm
###################################################
fit.stat <- fitStatisticalFactorModel(data=ret,
                                      k= "bn")
names(fit.stat)


###################################################
### code chunk number 18: stat.factor.returns
###################################################
fit.stat$k


###################################################
### code chunk number 19: plot.fit.stat
###################################################
plot(fit.stat,which.plot=2)


###################################################
### code chunk number 20: fig2
###################################################
plot(fit.stat,which.plot=2)


###################################################
### code chunk number 21: plot.fit.stat2
###################################################
plot(fit.stat,which.plot=1)


###################################################
### code chunk number 22: fig3
###################################################
plot(fit.stat,which.plot=1)


###################################################
### code chunk number 23: common.factors
###################################################
data(CommonFactors)
names(factors.Q)


###################################################
### code chunk number 24: merge
###################################################
ts.factors <- xts(factors.Q,as.yearqtr(index(factors.Q),format="%Y-%m-%d")) 
ts.data <- na.omit(merge(ret,ts.factors))


###################################################
### code chunk number 25: fit.ts
###################################################
fit.time <- fitTimeSeriesFactorModel(assets.names=tic,
                                     factors.names=c("SP500","Term.Spread","dVIX"),
                                     data=ts.data,fit.method="OLS")


###################################################
### code chunk number 26: single.asset.fit
###################################################
fit.time$asset.fit$AA


###################################################
### code chunk number 27: fit.ts
###################################################
fit.time2 <- fitTimeSeriesFactorModel(assets.names=tic,
                                      factors.names=names(ts.factors),
                                      data=ts.data,fit.method="OLS",
                                      variable.selection = "stepwise")


###################################################
### code chunk number 28: AA.fit
###################################################
fit.time2$asset.fit$AA


###################################################
### code chunk number 29: risk.decomp
###################################################
data.rd <- cbind(ret[,"AA"],fit.stat$factors,
                 fit.stat$residuals[,"AA"]/sqrt(fit.stat$resid.variance["AA"]))
var.decp <- factorModelVaRDecomposition(data.rd,fit.stat$loadings[,"AA"],
                            fit.stat$resid.variance["AA"],tail.prob=0.05,
                             VaR.method="gaussian")
names(var.decp)


###################################################
### code chunk number 30: mct.var
###################################################
var.decp$VaR.fm
var.decp$cVaR.fm


###################################################
### code chunk number 31: plot.rb.stat
###################################################
plot(fit.stat,which.plot=8,legend.text=TRUE, args.legend=list(x="topright"),
     VaR.method="gaussian")


###################################################
### code chunk number 32: fig4
###################################################
plot(fit.stat,which.plot=8,legend.text=TRUE, args.legend=list(x="topright"),
     VaR.method="gaussian")


###################################################
### code chunk number 33: port.var
###################################################
VaR(R=ret,method="gaussian",portfolio_method="component")


###################################################
### code chunk number 34: ts.attr
###################################################
ts.attr <- factorModelPerformanceAttribution(fit.time)
names(ts.attr)


